/*************************************************************************
 * Travis Veldkamp 02/16/2002                                            *
 * veldkamp@eudoramail.com                                               *
 * Copyright (C) 2002  Travis Veldkamp                                   *
 *                                                                       *
 * VERY simple routine for parsing the command line                      *
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include "cmd_line.h"
#include "util.h"

/****************************************************
 * Do a simple-minded parse of the command line     *
 *                                                  *
 * Returns: 0 if completed w/o error, -1 on error   *
 ****************************************************/
int parse_cmdline(int argc, char **argv, struct cmdline_struct *settings) {

   int i;
   FILE *temp_stream;

   settings->showhelp = 0;
   settings->verbose = VERBOSE_NORMAL;
   settings->generate_config_file = 0;
   settings->config_file[0] = '\0';
   settings->program_board = 1;  /*default to 1st board*/
   settings->run_detection = 0;  /*default is no board detection*/
   settings->no_program = 0;     /*default is to try to program*/
 
   for (i=1; i<argc;i++) {
      if ((strcmp(argv[i], HELP_FLAG1) == 0) || 
          (strcmp(argv[i], HELP_FLAG2) == 0)) {

         settings->showhelp = 1;
	 continue;
      } 

      if ((strcmp(argv[i], VERBOSE_FLAG1) == 0) ||
          (strcmp(argv[i], VERBOSE_FLAG2) == 0)) {

         settings->verbose = VERBOSE_VERBOSE;
	 continue;
      }

      if ((strcmp(argv[i], QUIET_FLAG1) == 0) ||
          (strcmp(argv[i], QUIET_FLAG2) == 0)) {
         settings->verbose = VERBOSE_QUIET;
	 continue;
      }

      if ((strcmp(argv[i], BOARD_FLAG1) == 0) ||
          (strcmp(argv[i], BOARD_FLAG2) == 0)) {

         /*The user is specifying which board to use*/
         i++;

	 if (argc <= i) {
	    printf("No board specified for %s\n", argv[i-1]);
	    return -1;
	 } else {
            settings->program_board = atoi(argv[i]);
            if ((settings->program_board < 1) || (settings->program_board > 2)) {
               printf("**Invalid board specified: %s.  Must be 1 or 2.**\n", 
                      argv[i]);
               return -1;
            }
	 }
	 continue;
      }

      if ((strcmp(argv[i], NO_PGM_FLAG1) == 0) ||
          (strcmp(argv[i], NO_PGM_FLAG2) == 0)) {

         /*User is specifying no programming cycle*/
	 /*This is probably only useful when combined*/
	 /*with the detect flag*/
         settings->no_program = 1;
         continue;
      }

      if ((strcmp(argv[i], GEN_CFG_FLAG1) == 0) ||
          (strcmp(argv[i], GEN_CFG_FLAG2) == 0)) {

         /*User has asked for a default config file to be built*/
         settings->generate_config_file = 1;

	 i++;

         /*The next entry should be the specified file*/
         if (argc > i) {
            /*Maybe we should check if file already exists so we don't 
              clobber anything*/
            strncpy(settings->config_file, argv[i], CFG_FILENAME_LEN);
            i++;
         } else {
            printf("**Error: No output file specified for %s**\n", 
	           argv[i-1]);
            return -1;
         }
	 continue;
      }

      if ((strcmp(argv[i], CONFIG_FLAG1) == 0) ||
          (strcmp(argv[i], CONFIG_FLAG2) == 0)) {

         /*The next entry should be the specified file*/
	 i++;

         if (argc > i) {
            /*verify that the file exists*/
            if ((temp_stream = fopen(argv[i], "r")) == NULL) {
               printf("**Error: config file doesn't exist: %s**\n",
                      argv[i]);
               return -1;
            } else {
               fclose(temp_stream);
               strncpy(settings->config_file, argv[i], CFG_FILENAME_LEN);
            }
         } else {
            printf("**Error: No output file specified for %s**\n", argv[i-1]);
            return -1;
         }
	 continue;
      }

      if ((strcmp(argv[i], DETECT_FLAG1) == 0) ||
          (strcmp(argv[i], DETECT_FLAG2) == 0)) {
         settings->run_detection = 1;
	 continue;
      }

      /*If we get to this point, we didn't match the argument with anything*/
      printf("**Error: unknown command line argument found: %s**\n", argv[i]);
      return -1;
   }
   return 0;
}
