/*************************************************************************
 * Travis Veldkamp 02/14/2002                                            *
 *                                                                       *
 * This file contains routines for controlling a keyboard attached to a  *
 * terminal.  In particular, it provides routines for controlling the    *
 * keyboard LEDs                                                         *
 *                                                                       *
 * There are now 2 SendLed functions - one for direct port access (new)  *
 * and the original one that uses the ioctl.  The original (ioctl) one   *
 * is better in that it's much more portable and uses the keyboard       *
 * driver.  However, the kernel has some optimization which prevents     *
 * duplicate operations from happening.  So, for example, if you use the *
 * ioctl interface to request turning on the caps lock LED twice, it will*
 * notice that it's already on and only send it once.  David Heremans    *
 * has a simple patch that will correct this behavior, but it requires   *
 * re-compiling your kernel.  See kbd_patch.txt for more information     *
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include "kbd_util.h"


/****************************************
 *Send a 3 bit code to set LEDs         *
 *without doing a read/modify/write     *
 *returns: 0 for success, -1 for error  *
 ****************************************/
#ifdef ARCH_OTHER
int SendLed(int console_fd, int code) {

   /* Set the LED states... */
   if (ioctl(console_fd, KDSETLED, code) < 0) {
      perror( "Error setting led state" );
      return -1;
   }
   
   /*Normal completion*/
   return 0;
}
#endif

/****************************************
 *Send a 3 bit code to set LEDs         *
 *without doing a read/modify/write     *
 *returns: 0 for success, -1 for error  *
 ****************************************/
#ifdef ARCH_i386
int SendLed(int console_fd, int code) {

   /*Ask for permission to use the data port*/
   if (ioperm(KBD_DATA_REG, 1, 1)) {
      perror("ioperm");
      return -1;
   }
 
   /* 0x0ED tells keyboard that next byte is for setting LEDs */
   outb(0xED, KBD_DATA_REG);
   usleep(600);

   /* Now, send led code.  All bits except b0..b2 MUST be 0 */
   /* Set a bit high to turn on LED, low to turn off */
   outb((code & 0x07), KBD_DATA_REG);

   /*relinquish access to the data port*/
   if (ioperm(KBD_DATA_REG, 1, 0)) {
      perror("ioperm: giving up failed");
      return -1;
   }

   return 0;
}
#endif
/*****************************************
 * Check if specified LED is on or off   *
 * return 1 if on, 0 if off              *
 *****************************************/
int CheckLed(int console_fd, int led) {

   char ledval;

   if (ioctl(console_fd, KDGETLED, &ledval) < 0) {
      perror( "Error getting led state" );
      exit(1);
   }

   if (ledval & led) 
      return 1;
   else
      return 0;
}
